// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef LEB_NETWORK_CONVERTER_H264_CONVERTER_H_
#define LEB_NETWORK_CONVERTER_H264_CONVERTER_H_

#include "leb_network/base/copy_on_write_buffer.h"
#include "leb_network/base/unwrapper.h"
#include "leb_network/base/throtter.h"
#include "leb_network/converter/base_converter.h"
#include "leb_network/leb_defines.h"

namespace liteav {
namespace lebnetwork {

class H264Converter : public BaseConverter<EncodedVideoFrame> {
 public:
  H264Converter();
  ~H264Converter();

  void Append(const RtpPacket& packet) override;

  std::vector<std::unique_ptr<EncodedVideoFrame>> Convert() override;

 private:
  // Return true if NAL units contain video data (not only contain SPS
  // and PPS)
  bool StapAToAnnexB(const uint8_t* stapa, size_t size,
                     trtc::CopyOnWriteBuffer* annexb, bool* is_key_frame);

  // Return true if the FU-A packet is the last packet of NAL unit (except SPS
  // and PPS)
  bool FuAToAnnexB(const uint8_t* fua, size_t size,
                   trtc::CopyOnWriteBuffer* annexb, bool* is_key_frame);

  liteav::Unwrapper unwrapper_seqnum_;
  liteav::Unwrapper unwrapper_dts_;
  trtc::CopyOnWriteBuffer nal_buffer_;
  bool is_key_frame_;
  uint8_t nal_head_;
  std::vector<std::unique_ptr<EncodedVideoFrame>> frames_;
  base::Throtter throtter_;
};

}  // namespace lebnetwork
}  // namespace liteav

#endif  // LEB_NETWORK_CONVERTER_H264_CONVERTER_H_
