// Copyright (c) 2021 Tencent. All rights reserved.

#include "leb_network/converter/opus_converter.h"

namespace liteav {
namespace lebnetwork {

OpusConverter::OpusConverter(const SdpAudioFormat& format) : format_(format),
      unwrapper_dts_(0, std::numeric_limits<uint32_t>::max())  {}

OpusConverter::~OpusConverter() = default;

void OpusConverter::Append(const RtpPacket& packet) {
  std::unique_ptr<EncodedAudioFrame> frame(new EncodedAudioFrame());

  frame->codec_type = AudioCodecType::kOpus;
  frame->bits_per_sample = 16;
  frame->num_channels = format_.num_channels;
  frame->pts = (int64_t)unwrapper_dts_.Unwrap(static_cast<uint64_t>(packet.decoding_timestamp));
  frame->sample_rate = format_.clockrate_hz;
  frame->data.SetData(packet.payload(), packet.payload_size());
  frame->frame_type = AudioFrameType::kOpus;
  frame->output_delay_ms = packet.output_delay_ms;

  frames_.push_back(std::move(frame));
}

std::vector<std::unique_ptr<EncodedAudioFrame>> OpusConverter::Convert() {
  return std::move(frames_);
}

}  // namespace lebnetwork
}  // namespace liteav
