// Copyright (c) 2021 Tencent. All rights reserved.

#include "leb_network/internal_defines.h"

#include <stdio.h>

namespace {

using LogObserver = liteav::lebnetwork::LEBConnection::LogObserver;
using LogLevel = liteav::lebnetwork::LogLevel;

LogObserver* g_log_observer = nullptr;
LogLevel g_log_level = LogLevel::kInfo;

}  // namespace

namespace liteav {
namespace lebnetwork {

MediaTransportLogLevel LogLevelToMediaTransportLogLevel(LogLevel level) {
  switch (level) {
    case LogLevel::kDebug:
      return MediaTransportLogLevel::MediaTransportLogLevelDebug;
    case LogLevel::kInfo:
      return MediaTransportLogLevel::MediaTransportLogLevelInfo;
    case LogLevel::kWarning:
      return MediaTransportLogLevel::MediaTransportLogLevelWarning;
    case LogLevel::kError:
      return MediaTransportLogLevel::MediaTransportLogLevelError;
  }

  return MediaTransportLogLevel::MediaTransportLogLevelNone;
}

LogLevel MediaTransportLogLevelToLogLevel(MediaTransportLogLevel level) {
  switch (level) {
    case MediaTransportLogLevel::MediaTransportLogLevelDebug:
      return LogLevel::kDebug;
    case MediaTransportLogLevel::MediaTransportLogLevelInfo:
      return LogLevel::kInfo;
    case MediaTransportLogLevel::MediaTransportLogLevelWarning:
      return LogLevel::kWarning;
    case MediaTransportLogLevel::MediaTransportLogLevelError:
      return LogLevel::kError;
    default:
      return LogLevel::kDebug;
  }

  return LogLevel::kDebug;
}

void SetLogObserver(LEBConnection::LogObserver* observer, LogLevel level) {
  g_log_observer = observer;
  g_log_level = level;
}

void NotifyLog(const char* tag, LogLevel level, const char* msg) {
  if (level < g_log_level) {
    return;
  }
  if (g_log_observer) {
    g_log_observer->OnLog(tag, level, msg);
  }
}

Error RTCErrorToError(const RTCError& error) {
  Error e;
  e.set_error_code_ex(error.error_code(), error.error_code_ex());
  e.set_message(error.message());
  e.set_type(error.type());
  return e;
}

void CopyAudioInfo(const RtpPacket& packet, const SdpAudioFormat& format,
                   EncodedAudioFrame* frame) {
  frame->codec_type = AudioCodecType::kAac;
  // TODO(yaobohua): how do i known the real bits it is?
  frame->bits_per_sample = 16;
  frame->num_channels = format.num_channels;
  frame->pts = packet.decoding_timestamp;
  frame->sample_rate = format.clockrate_hz;
}

}  // namespace lebnetwork
}  // namespace liteav
