// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef LEB_NETWORK_INTERNAL_DEFINES_H_
#define LEB_NETWORK_INTERNAL_DEFINES_H_

#include <stdio.h>

#include <memory>

#include "include/peer_connection.h"
#include "include/rtc_configuration.h"
#include "include/rtc_error.h"
#include "include/rtc_receiver.h"
#include "include/rtp_packet_sink_interface.h"
#include "leb_network/leb_connection.h"
#include "leb_network/leb_defines.h"

using MediaTransportLogLevel = media_transport::MediaTransportLogLevel;
using RTCPeerConnection = media_transport::RTCPeerConnection;
using RTCPeerConnectionDelegate = media_transport::RTCPeerConnectionDelegate;
using RTCPeerConnectionState = media_transport::RTCPeerConnectionState;
using RTCStatsReport = media_transport::RTCStatsReport;
using RTCIceCandidate = media_transport::RTCIceCandidate;
using RTCRtpReceiver = media_transport::RTCRtpReceiver;
using RTCSessionDescriptionDelegate =
    media_transport::RTCSessionDescriptionDelegate;
using RTCSessionDescription = media_transport::RTCSessionDescription;
using RTCError = media_transport::RTCError;
using RTCRtpReceiverDelegate = media_transport::RTCRtpReceiverDelegate;
using RtpPacketSinkInterface = media_transport::RtpPacketSinkInterface;
using RTCRtpMediaType = media_transport::RTCRtpMediaType;
using RtpPacket = media_transport::RtpPacket;
using RtpMetaData = media_transport::RtpMetaData;
using SdpAudioFormat = media_transport::SdpAudioFormat;
using SdpVideoFormat = media_transport::SdpVideoFormat;
using RTCSdpType = media_transport::RTCSdpType;

namespace liteav {
namespace lebnetwork {

enum NaluType : uint8_t {
  // 1-23  都是单个 NAL 包
  kSlice = 1,
  kIdr = 5,
  kSei = 6,
  kSps = 7,
  kPps = 8,
  kAud = 9,
  kEndOfSequence = 10,
  kEndOfStream = 11,
  kFiller = 12,
  // 单一时间的组合包
  kStapA = 24,
  // 单一时间的组合包
  kStapB = 25,
  // 多个时间的组合包
  kMtap16 = 26,
  // 多个时间的组合包
  kMtap24 = 27,
  // 分片的单元
  kFuA = 28,
  // 分片的单元
  kFuB = 29
};

enum H265NaluType : uint8_t {
  kH265Idr = 19,
  kH265Cra = 21,
  kH265Vps = 32,
  kH265Sps = 33,
  kH265Pps = 34,
  kH265Sei = 39,
  // 多个时间的组合包
  kH265Ap = 48,
  // 分片的单元
  kH265Fu = 49,
};

MediaTransportLogLevel LogLevelToMediaTransportLogLevel(LogLevel level);

LogLevel MediaTransportLogLevelToLogLevel(MediaTransportLogLevel level);

void SetLogObserver(LEBConnection::LogObserver* observer, LogLevel level);

void NotifyLog(const char* tag, LogLevel level, const char* msg);

template <typename... Args>
void NotifyLog(const char* tag, LogLevel level, const char* format, Args... args) {
  size_t size = snprintf(nullptr, 0, format, args...) + 1;
  std::unique_ptr<char[]> buf(new char[size]);
  snprintf(buf.get(), size, format, args...);
  NotifyLog(tag, level, buf.get());
}

Error RTCErrorToError(const RTCError& error);

void CopyAudioInfo(const RtpPacket& packet, const SdpAudioFormat& format,
                   EncodedAudioFrame* frame);

}  // namespace lebnetwork
}  // namespace liteav

#endif  // LEB_NETWORK_INTERNAL_DEFINES_H_
