// Copyright (c) 2021 Tencent. All rights reserved.

#include "leb_network/leb_connection.h"

#include <memory>

#include "include/rtc_configuration.h"
#include "leb_network/base/no_destructor.h"
#include "leb_network/leb_connection_impl.h"

namespace {

class InnerLogObserver
    : public media_transport::RTCFileLogger::RTCLogInterface {
 public:
  void OnLog(const char* tag, MediaTransportLogLevel level,
             const char* msg) override {
    liteav::lebnetwork::NotifyLog(tag,
        liteav::lebnetwork::MediaTransportLogLevelToLogLevel(level), msg);
  }
};

std::shared_ptr<InnerLogObserver>& GetLogObserverInstance() {
  static base::NoDestructor<std::shared_ptr<InnerLogObserver>> instance;
  return *instance;
}

}  // namespace

namespace liteav {
namespace lebnetwork {

void LEBConnection::Initialize(LogObserver* observer, LogLevel level) {
  liteav::lebnetwork::SetLogObserver(observer, level);
  std::shared_ptr<InnerLogObserver>& log_observer = GetLogObserverInstance();
  log_observer.reset(new InnerLogObserver());
  auto logger = media_transport::RTCFileLogger::getInstance(nullptr, 0);
  logger->setLogLevel(media_transport::MediaTransportLogLevel::MediaTransportLogLevelDebug);
  logger->setLogInterface(log_observer);
  logger->start();
}

void LEBConnection::Uninitialize() {
  auto logger = media_transport::RTCFileLogger::getInstance(nullptr, 0);
  logger->stop();
  std::shared_ptr<InnerLogObserver>& log_observer = GetLogObserverInstance();
  log_observer.reset();
  liteav::lebnetwork::SetLogObserver(nullptr, LogLevel::kInfo);
}

std::unique_ptr<LEBConnection> LEBConnection::Create(Client* client) {
  std::unique_ptr<LEBConnection> pc =
      std::unique_ptr<LEBConnectionImpl>(new LEBConnectionImpl(client));
  return pc;
}

}  // namespace lebnetwork
}  // namespace liteav
