// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef LEB_NETWORK_LEB_CONNECTION_H_
#define LEB_NETWORK_LEB_CONNECTION_H_

#include <memory>
#include <string>
#include <utility>

#include "leb_network/leb_defines.h"

#ifdef _MSC_VER
#pragma warning(disable : 4716)
#endif  // _MSC_VER

namespace liteav {
namespace lebnetwork {

class LEBConnection {
 public:
  class Client {
   public:
    virtual ~Client() = default;
    virtual void OnSdpInfo(SdpInfo& info) = 0;
    virtual void OnVideoInfo(VideoInfo& info) = 0;
    virtual void OnAudioInfo(AudioInfo& info) = 0;
    virtual void OnEncodedVideo(std::unique_ptr<EncodedVideoFrame> frame) = 0;
    virtual void OnEncodedAudio(std::unique_ptr<EncodedAudioFrame> frame) = 0;
    virtual void OnMetadata(std::unique_ptr<Metadata> data) = 0;

    virtual void OnError(const Error& error) = 0;
    virtual void OnNetState(NetState state) = 0;
    virtual void OnStats(const Stats& stats) = 0;

    virtual void SetHasVideo() {}
    virtual void SetHasAudio() {}
    virtual bool HasVideo() {}
    virtual bool HasAudio() {}
    virtual bool HasVideoInformed() {}
    virtual bool HasAudioInformed() {}
  };

  class LogObserver {
   public:
    virtual ~LogObserver() = default;
    virtual void OnLog(const char* tag, LogLevel level, const char* msg) = 0;
  };

  struct Configuration {
    // 拉流地址，如: "webrtc://5664.liveplay.myqcloud.com/live/5664_harchar1"
    std::string stream_url;
    
    std::string remote_address = "webrtc.liveplay.myqcloud.com";
    
    bool enable_udp_signal = true;

    // 视频开关，true 表示拉取视频流
    bool receive_video = true;

    // 音频开关，true 表示拉取音频流
    bool receive_audio = true;

    // 加密开关
    bool enable_encryption = false;

    // 是否开启 AAC 选项，打开且推流音频是 AAC，CDN 下发 AAC，否则默认音频是
    // Opus
    bool enable_aac = true;

    // Flex FEC 开关
    bool enable_flex_fec = true;

    // 统计回调周期
    int stats_period_ms = 5000;

    // 音频补包开关
    bool enable_audio_plc = false;

    // 0RTT开关
    bool enable_0rtt = false;

    // 控制首个GOP回退数据下发量，一般取播放器追帧buffer阈值
    int min_delay_ms = 1000;
  };

  static void Initialize(LogObserver* observer, LogLevel level);

  static void Uninitialize();

  static std::unique_ptr<LEBConnection> Create(Client* client);

  virtual bool Start(const Configuration& config) = 0;

  // Used only for http signal
  virtual void SetRemoteSDP(const SdpInfo& info) = 0;

  virtual void Stop() = 0;

  // Stats will callback asynchronously through Client::OnStats
  virtual void GetStats() = 0;

  virtual ~LEBConnection() = default;
};

}  // namespace lebnetwork
}  // namespace liteav

#endif  // LEB_NETWORK_LEB_CONNECTION_H_
