// Copyright (c) 2021 Tencent. All rights reserved.

#include "leb_network/metadata_channel_receiver.h"

#include <memory>

#include "leb_network/leb_defines.h"

namespace liteav {
namespace lebnetwork {

MetadataChannelReceiver::MetadataChannelReceiver(
    std::shared_ptr<RTCPeerConnection> pc, LEBConnection::Client* client)
    : ChannelReceiver(pc, client) {}

MetadataChannelReceiver::~MetadataChannelReceiver() = default;

// Implement RTCRtpReceiverDelegate
void MetadataChannelReceiver::OnReceiveFirstPacket(RTCRtpMediaType mediaType) {}

int MetadataChannelReceiver::getEndSeqNumOfLastDecodedFrame(
    RTCRtpMediaType mediaType, uint16_t* sequence_number, uint32_t* timestamp) {
  return -1;
}

void MetadataChannelReceiver::OnReceiveAudioCodecs(
    const std::map<int, SdpAudioFormat>& codecs) {}

void MetadataChannelReceiver::OnReceiveVideoCodecs(
    const std::map<int, SdpVideoFormat>& codecs) {}

// Implement RtpPacketSinkInterface
void MetadataChannelReceiver::OnRtpPacket(const RtpPacket& packet,
                                          RTCRtpMediaType mediaType) {}

void MetadataChannelReceiver::OnRtpMetaDataPacket(const RtpMetaData& packet,
                                                  RTCRtpMediaType mediaType) {
  std::unique_ptr<Metadata> data(new Metadata());
  switch (mediaType) {
    case RTCRtpMediaType::RTCRtpMediaTypeAudio:
      data->type = MetadataType::kAudio;
      break;
    case RTCRtpMediaType::RTCRtpMediaTypeVideo:
      data->type = MetadataType::kVideo;
      break;
#ifdef WEBRTC_BUILD_DATA_CHANNEL
    case RTCRtpMediaType::RTCRtpMediaTypeData:
      data->type = MetadataType::kData;
      break;
#endif
  }

  auto md = packet.getByteMetaData();
  data->data.SetData(&(md[0]), md.size());

  client_->OnMetadata(std::move(data));
}

void MetadataChannelReceiver::OnMediaPacket(const void* data, int length,
                                            int64_t pts,
                                            RTCRtpMediaType mediaType,
                                            bool iscodec) {}

}  // namespace lebnetwork
}  // namespace liteav
