// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef LEB_NETWORK_METADATA_CHANNEL_RECEIVER_H_
#define LEB_NETWORK_METADATA_CHANNEL_RECEIVER_H_

#include "leb_network/channel_receiver.h"
#include "leb_network/internal_defines.h"

namespace liteav {
namespace lebnetwork {

class MetadataChannelReceiver : public ChannelReceiver {
 public:
  MetadataChannelReceiver(std::shared_ptr<RTCPeerConnection> pc,
                          LEBConnection::Client* client);
  ~MetadataChannelReceiver() override;

  // Implement RTCRtpReceiverDelegate
  void OnReceiveFirstPacket(RTCRtpMediaType mediaType) override;

  int getEndSeqNumOfLastDecodedFrame(RTCRtpMediaType mediaType,
                                     uint16_t* sequence_number,
                                     uint32_t* timestamp) override;

  void OnReceiveAudioCodecs(
      const std::map<int, SdpAudioFormat>& codecs) override;

  void OnReceiveVideoCodecs(
      const std::map<int, SdpVideoFormat>& codecs) override;

  // Implement RtpPacketSinkInterface
  void OnRtpPacket(const RtpPacket& packet, RTCRtpMediaType mediaType) override;

  void OnRtpMetaDataPacket(const RtpMetaData& packet,
                           RTCRtpMediaType mediaType) override;

  void OnMediaPacket(const void* data, int length, int64_t pts,
                     RTCRtpMediaType mediaType, bool iscodec) override;
};

}  // namespace lebnetwork
}  // namespace liteav

#endif  // LEB_NETWORK_METADATA_CHANNEL_RECEIVER_H_
