// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef LEB_NETWORK_SESSION_DESCRIPTION_EXCHANGE_H_
#define LEB_NETWORK_SESSION_DESCRIPTION_EXCHANGE_H_

#include <memory>

#include "include/peer_connection.h"
#include "leb_network/internal_defines.h"
#include "leb_network/leb_connection.h"

namespace liteav {
namespace lebnetwork {

class SessionDescriptionExchange
    : public RTCSessionDescriptionDelegate,
      public std::enable_shared_from_this<SessionDescriptionExchange> {
 public:
  class SDPExchangeObserver {
   public:
    virtual ~SDPExchangeObserver() = default;
    virtual void OnOfferCreated(RTCSessionDescription& sdp) = 0;
    virtual void OnSignaling(const std::string& server_ip,
                             const int64_t dns_cost_ms) = 0;
  };

  SessionDescriptionExchange(std::shared_ptr<RTCPeerConnection> pc,
                             SDPExchangeObserver* sdp_observer,
                             LEBConnection::Client* error_observer);
  ~SessionDescriptionExchange();

  void CreateOffer();

  void SetRemoteSDP(RTCSessionDescription& sdp);

  void GetSignalServerInfo();

 private:
  void NotifyError(const RTCError* error);

  void onCreateSessionDescription(RTCPeerConnection* pc,
                                  const RTCSessionDescription* sdp,
                                  const RTCError* error,
                                  void* user_data) override;

  void onSignalingResolve(RTCPeerConnection* pc, const std::string& server_ip,
                          const int64_t dns_cost_ms) override;

  void onCompletion(RTCPeerConnection* pc, const RTCError* error,
                    void* user_data) override;

 private:
  std::shared_ptr<RTCPeerConnection> pc_;
  LEBConnection::Client* error_observer_;
  SDPExchangeObserver* sdp_observer_;
  RTCSessionDescription local_description_;
  RTCSessionDescription remote_description_;
};

}  // namespace lebnetwork
}  // namespace liteav

#endif  // LEB_NETWORK_SESSION_DESCRIPTION_EXCHANGE_H_
