// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef LEB_NETWORK_VIDEO_CHANNEL_RECEIVER_H_
#define LEB_NETWORK_VIDEO_CHANNEL_RECEIVER_H_

#include "leb_network/base/throtter.h"
#include "leb_network/channel_receiver.h"
#include "leb_network/converter/base_converter.h"
#include "leb_network/internal_defines.h"

namespace liteav {
namespace lebnetwork {

class VideoChannelReceiver : public ChannelReceiver {
 public:
  VideoChannelReceiver(std::shared_ptr<RTCPeerConnection> pc,
                       LEBConnection::Client* client);
  ~VideoChannelReceiver() override;

  // Implement RTCRtpReceiverDelegate
  void OnReceiveFirstPacket(RTCRtpMediaType mediaType) override;

  int getEndSeqNumOfLastDecodedFrame(RTCRtpMediaType mediaType,
                                     uint16_t* sequence_number,
                                     uint32_t* timestamp) override;

  void OnReceiveAudioCodecs(
      const std::map<int, SdpAudioFormat>& codecs) override;

  void OnReceiveVideoCodecs(
      const std::map<int, SdpVideoFormat>& codecs) override;

  void OnReceiveVideoSpropParameterSets(const uint8_t* extradata, size_t size) override;

  // Implement RtpPacketSinkInterface
  void OnRtpPacket(const RtpPacket& packet, RTCRtpMediaType mediaType) override;

  void OnRtpMetaDataPacket(const RtpMetaData& packet,
                           RTCRtpMediaType mediaType) override;

  void OnMediaPacket(const void* data, int length, int64_t pts,
                     RTCRtpMediaType mediaType, bool iscodec) override;

 private:
  void EnsureConverterGenerated(uint8_t payload_type);

  std::map<int, SdpVideoFormat> formats_;
  std::unique_ptr<BaseConverter<EncodedVideoFrame>> converter_;
  base::Throtter throtter_;
  bool hasInformed_;
};

}  // namespace lebnetwork
}  // namespace liteav

#endif  // LEB_NETWORK_VIDEO_CHANNEL_RECEIVER_H_
