// Copyright (c) 2020 Tencent. All rights reserved.

#ifndef MEDIATRANSPORTTEST_DEMO3_LEB_CONNECTION_TEST_H_
#define MEDIATRANSPORTTEST_DEMO3_LEB_CONNECTION_TEST_H_

#include <stdio.h>
#include <inttypes.h>
#include <memory>

#include "leb_network/leb_connection.h"

namespace liteav {
namespace lebnetwork {

class LEBConnectionTest : public LEBConnection::Client,
                          public LEBConnection::LogObserver {
 public:
  explicit LEBConnectionTest(const std::string& store_dir);
  ~LEBConnectionTest();

  void Start(const LEBConnection::Configuration& config);

  void Stop();

  void GetStats();

  // Implement LEBConnection::Client
  void OnSdpInfo(SdpInfo& info) override {}
  void OnVideoInfo(VideoInfo& info) override {}
  void OnAudioInfo(AudioInfo& info) override {}
  void OnEncodedVideo(std::unique_ptr<EncodedVideoFrame> frame) override;
  void OnEncodedAudio(std::unique_ptr<EncodedAudioFrame> frame) override;
  void OnMetadata(std::unique_ptr<Metadata> data) override;

  void OnError(const Error& error) override;
  void OnNetState(NetState state) override;
  void OnStats(const Stats& stats) override;

  // Implement LEBConnection::LogObserver
  void OnLog(const char* tag, LogLevel level, const char* msg) override;

 private:
  std::string store_dir_;
  std::unique_ptr<LEBConnection> pc_;
  FILE* audio_file_;
  FILE* video_file_;
};
}  // namespace lebnetwork
}  // namespace liteav

#endif  // MEDIATRANSPORTTEST_DEMO3_LEB_CONNECTION_TEST_H_
