// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef RTC_RECEIVER_H
#define RTC_RECEIVER_H
#include <string>
#include <stdio.h>
#include <map>
#include <mutex>
#include "rtc_macros.h"
#include "rtp_packet_sink_interface.h"

namespace media_transport {



class RTCRtpReceiver;
//class RtpPacketSinkInterface;


/**
 * RTP Receiver回调接口
 * 
 */
class RTCRtpReceiverDelegate {
 public:
  virtual ~RTCRtpReceiverDelegate() = default;

  /** 收到音视频第一个RTP包回调 */
  virtual void OnReceiveFirstPacket(RTCRtpMediaType mediaType) = 0;
  
  /** 通过回调通知sdk外部解码到第几帧  deprecated*/
  virtual int getEndSeqNumOfLastDecodedFrame(RTCRtpMediaType mediaType, uint16_t *sequence_number, uint32_t *timestamp) = 0;
  
  /** setRemoteSdp后音频解码器信息的回调 */
  virtual void OnReceiveAudioCodecs(const std::map<int,
                                    media_transport::SdpAudioFormat>& codecs) = 0;
  
  /** setRemoteSdp后视频解码器信息的回调 */
  virtual void OnReceiveVideoCodecs(const std::map<int,
                                    media_transport::SdpVideoFormat>& codecs) = 0;
  
  virtual void OnReceiveVideoSpropParameterSets(const uint8_t* extradata, size_t size) {}
  
  /*deprecated*/
  virtual void OnReceiveVideoCodecs(std::map<uint8_t, media_transport::VideoCodecType> &payload_type,
                                    std::map<uint8_t, std::map<std::string, std::string> > &pt_codec_params){ }
};

/**
 * RTP Receiver
 * 
 */
class RTC_EXPORT_API RTCRtpReceiver {
public:
  RTCRtpReceiver(RTCRtpMediaType type);
  ~RTCRtpReceiver();
  
  /**  设置RTCRtpReceiverDelegate */
  void setDelegate(std::weak_ptr<RTCRtpReceiverDelegate> delegate);
  
  std::weak_ptr<RTCRtpReceiverDelegate> getDelegate();
  
  /**  设置RTP sink */
  void setSink(std::weak_ptr<RtpPacketSinkInterface> sink);
  
  std::weak_ptr<RtpPacketSinkInterface> getSink();
  
  RTCRtpMediaType getMediaType();

private:
  RTCRtpMediaType _mediaType;

private:
  std::mutex _pLock;
  
  std::weak_ptr<RTCRtpReceiverDelegate> _delegate;
  std::weak_ptr<RtpPacketSinkInterface> _sink;
};


}

#endif /* RTC_RECEIVER_H */
