// Copyright (c) 2021 Tencent. All rights reserved.

#ifndef RTP_PACKET_H
#define RTP_PACKET_H
#include "rtp_header.h"
#include "rtc_macros.h"
#include <vector>

namespace media_transport {

class RTC_EXPORT_API RtpPacket {
public:
  RtpPacket();
  virtual ~RtpPacket();
  
  virtual size_t payload_size() const;
  virtual const uint8_t* payload() const;
  
  virtual void setPayLoadBuffer(void *dataBuffer);
  
  virtual void setPayLoadBuffer(void *dataBuffer, size_t size);
  
  virtual void reset();

public:
    RTPHeader header;
    int64_t receive_time_ms;

    int32_t output_delay_ms;
    
    /*remote server ntp*/
    int64_t ntp_time_ms;
  
    /*flv 回源的原始时戳*/
    uint32_t decoding_timestamp;
    
    //rtp 分片开始
    bool is_first_packet_in_frame;

    //rtp 帧结束
    bool is_last_packet_in_frame;
    
    //rtp FuA 结束(slice分片结束)
    bool is_last_packet_in_slice;
  
private:
  RtpPacket& operator=(const RtpPacket&) = delete;
   RtpPacket(const RtpPacket&) = delete;

protected:
  void *_dataBuffer;
};

class RTC_EXPORT_API RtpMetaData {
public:
  RtpMetaData();
  virtual ~RtpMetaData();
  
  
  std::vector<uint8_t> getByteMetaData() const;
  
  void addByteMetaData(const uint8_t* medadataSegBegin, const uint8_t* medadataSegEnd);
  
private:
  std::vector<uint8_t> byte_metadata_;
};

}

#endif /* RTP_PACKET_H */
